const {expect} = require('chai')
const {setupServer} = require('msw/node')
const {setupPactMswAdapter} = require('@pactflow/pact-msw-adapter')
const {toQueryString} = require('@s-ui/js/lib/string')
const {writerFactory, mapProviders, getContractTests} = require('./utils.js')

module.exports = ({apiUrl, consumer, providers, fetcher, defaultOptions}) => {
  const server = setupServer()
  const pactMswAdapter = setupPactMswAdapter({
    server,
    options: {
      consumer,
      providers: mapProviders(providers),
      pactOutDir: './contract/_documents',
      excludeHeaders: ['x-powered-by']
    }
  })
  const tests = getContractTests({apiUrl, providers})

  describe('Consumer test contracts', () => {
    before(async () => {
      server.listen()
    })

    beforeEach(async () => {
      pactMswAdapter.newTest()
    })

    afterEach(async () => {
      pactMswAdapter.verifyTest()
      server.resetHandlers()
    })

    after(async () => {
      const writer = writerFactory(providers)

      await pactMswAdapter.writeToFile(writer)
      pactMswAdapter.clear()
      server.close()
    })

    tests.forEach(
      ({
        description,
        endpoint,
        query,
        body,
        method = 'get',
        handler,
        options = defaultOptions,
        response
      }) => {
        it(`makes ${description.toLowerCase()}`, async () => {
          server.use(handler)

          let url = `${apiUrl}${endpoint}`
          if (query) url = `${url}?${toQueryString(query)}`

          const {data} =
            method === 'post'
              ? await fetcher[method](url, body, options)
              : await fetcher[method](url, options)

          if (data) expect(data).to.deep.equal(response)
        })
      }
    )
  })
}
